@@ -13,7 +13,6 @@ import java.io.IOException;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
-import java.nio.file.Paths;
 import java.sql.Connection;
 import java.sql.DriverManager;
 import java.sql.PreparedStatement;
@@ -25,40 +24,50 @@ import java.sql.Types;
 import java.text.DateFormat;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
-import java.util.Calendar;
 import java.util.HashSet;
 import java.util.List;
-import java.util.Locale;
 import java.util.Set;
-import java.util.TimeZone;
 
 public class Main {
 
   /** Logger for this class. */
-  private static Logger log = LoggerFactory.getLogger(Main.class);
+  private static final Logger logger = LoggerFactory.getLogger(Main.class);
+
+  private static final String jdbcString = String.format(
+      "jdbc:postgresql://localhost/onionperf?user=%s&password=%s",
+      System.getProperty("metrics.dbuser", "metrics"),
+      System.getProperty("metrics.dbpass", "password"));
+
+  private static final File baseDir = new File(
+      org.torproject.metrics.stats.main.Main.modulesDir, "onionperf");
 
   /** Executes this data-processing module. */
   public static void main(String[] args) throws Exception {
-    log.info("Starting onionperf module.");
-    String dbUrlString = "jdbc:postgresql:onionperf";
-    Connection connection = connectToDatabase(dbUrlString);
+    logger.info("Starting onionperf module.");
+    Connection connection = connectToDatabase();
     importOnionPerfFiles(connection);
-    writeStatistics(Paths.get("stats", "torperf-1.1.csv"),
-        queryOnionPerf(connection));
-    writeStatistics(Paths.get("stats", "buildtimes.csv"),
+    writeStatistics(
+        new File(baseDir, "stats/onionperf-including-partials.csv").toPath(),
+        queryOnionperfIncludingPartials(connection));
+    writeStatistics(new File(baseDir, "stats/onionperf-failures.csv").toPath(),
+        queryOnionperfFailures(connection));
+    writeStatistics(new File(baseDir, "stats/buildtimes.csv").toPath(),
         queryBuildTimes(connection));
-    writeStatistics(Paths.get("stats", "latencies.csv"),
+    writeStatistics(new File(baseDir, "stats/latencies.csv").toPath(),
         queryLatencies(connection));
+    writeStatistics(
+        new File(baseDir, "stats/onionperf-throughput.csv").toPath(),
+        queryThroughput(connection));
     disconnectFromDatabase(connection);
-    log.info("Terminated onionperf module.");
+    logger.info("Terminated onionperf module.");
   }
 
-  private static Connection connectToDatabase(String jdbcString)
+  private static Connection connectToDatabase()
       throws SQLException {
-    log.info("Connecting to database.");
+    logger.info("Connecting to database.");
     Connection connection = DriverManager.getConnection(jdbcString);
     connection.setAutoCommit(false);
-    log.info("Successfully connected to database.");
+    logger.info("Successfully connected to database.");
     return connection;
   }
 
@@ -77,7 +86,10 @@ public class Main {
         + "dataperc60, dataperc70, dataperc80, dataperc90, dataperc100, "
         + "launch, used_at, timeout, quantile, circ_id, used_by, "
         + "endpointlocal, endpointproxy, endpointremote, hostnamelocal, "
-        + "hostnameremote, sourceaddress) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, "
+        + "hostnameremote, sourceaddress, partial10240, partial20480, "
+        + "partial51200, partial102400, partial204800, partial512000, "
+        + "partial1048576, partial2097152, partial5242880) "
+        + "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
         + "?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "
         + "?, ?, ?, ?, ?, ?)", Statement.RETURN_GENERATED_KEYS);
 
@@ -88,11 +100,12 @@ public class Main {
         "INSERT INTO buildtimes (measurement_id, position, buildtime, delta) "
             + "VALUES (?, ?, ?, ?)");
 
-    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
     DescriptorReader dr = DescriptorSourceFactory.createDescriptorReader();
     for (Descriptor d : dr.readDescriptors(
-        new File("../../shared/in/archive/torperf"),
-        new File("../../shared/in/recent/torperf"))) {
+        new File(org.torproject.metrics.stats.main.Main.descriptorsDir,
+            "archive/onionperf"),
+        new File(org.torproject.metrics.stats.main.Main.descriptorsDir,
+            "recent/onionperf"))) {
       if (!(d instanceof TorperfResult)) {
         continue;
       }
@@ -103,7 +116,7 @@ public class Main {
       psMeasurementsSelect.setString(1, truncatedSource);
       psMeasurementsSelect.setInt(2, tr.getFileSize());
       psMeasurementsSelect.setTimestamp(3,
-          new Timestamp(tr.getStartMillis()), calendar);
+          new Timestamp(tr.getStartMillis()));
       try (ResultSet rs = psMeasurementsSelect.executeQuery()) {
         if (rs.next()) {
           measurementId = rs.getInt(1);
@@ -114,7 +127,7 @@ public class Main {
         psMeasurementsInsert.setString(1, truncatedSource);
         psMeasurementsInsert.setInt(2, tr.getFileSize());
         psMeasurementsInsert.setTimestamp(3,
-            new Timestamp(tr.getStartMillis()), calendar);
+            new Timestamp(tr.getStartMillis()));
         long[] timestamps = new long[] { tr.getSocketMillis(),
             tr.getConnectMillis(), tr.getNegotiateMillis(),
             tr.getRequestMillis(), tr.getResponseMillis(),
@@ -148,13 +161,13 @@ public class Main {
           psMeasurementsInsert.setNull(26, Types.TIMESTAMP);
         } else {
           psMeasurementsInsert.setTimestamp(26,
-              new Timestamp(tr.getLaunchMillis()), calendar);
+              new Timestamp(tr.getLaunchMillis()));
         }
         if (tr.getUsedAtMillis() < 0L) {
           psMeasurementsInsert.setNull(27, Types.TIMESTAMP);
         } else {
           psMeasurementsInsert.setTimestamp(27,
-              new Timestamp(tr.getUsedAtMillis()), calendar);
+              new Timestamp(tr.getUsedAtMillis()));
         }
         if (tr.getTimeout() < 0L) {
           psMeasurementsInsert.setNull(28, Types.INTEGER);
@@ -188,6 +201,18 @@ public class Main {
                 truncateString(onionPerfStrings[j], 64));
           }
         }
+        int[] partialBytes = new int[] { 10240, 20480, 51200, 102400, 204800,
+            512000, 1048576, 2097152, 5242880 };
+        for (int i = 38, j = 0; j < partialBytes.length; i++, j++) {
+          if (null == tr.getPartials()
+              || !tr.getPartials().containsKey(partialBytes[j])) {
+            psMeasurementsInsert.setNull(i, Types.INTEGER);
+          } else {
+            psMeasurementsInsert.setInt(i,
+                (int) (tr.getPartials().get(partialBytes[j])
+                - tr.getStartMillis()));
+          }
+        }
         psMeasurementsInsert.execute();
         try (ResultSet rs = psMeasurementsInsert.getGeneratedKeys()) {
           if (rs.next()) {
@@ -231,28 +256,46 @@ public class Main {
     return originalString;
   }
 
-  static List<String> queryOnionPerf(Connection connection)
+  static List<String> queryOnionperfIncludingPartials(Connection connection)
       throws SQLException {
-    log.info("Querying statistics from database.");
+    logger.info("Querying statistics including partials from database.");
     List<String> statistics = new ArrayList<>();
     statistics
-        .add("date,filesize,source,server,q1,md,q3,timeouts,failures,requests");
+        .add("date,filesize,source,server,q1,md,q3");
     Statement st = connection.createStatement();
-    String queryString = "SELECT date, filesize, source, server, q1, md, q3, "
-        + "timeouts, failures, requests FROM onionperf";
-    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
-    dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
-    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+    String queryString = "SELECT date, filesize, source, server, q1, md, q3 "
+        + " FROM onionperf_including_partials";
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
     try (ResultSet rs = st.executeQuery(queryString)) {
       while (rs.next()) {
-        statistics.add(String.format("%s,%d,%s,%s,%.0f,%.0f,%.0f,%d,%d,%d",
-            dateFormat.format(rs.getDate("date", calendar)),
+        statistics.add(String.format("%s,%d,%s,%s,%.0f,%.0f,%.0f",
+            dateFormat.format(rs.getDate("date")),
             rs.getInt("filesize"),
-            emptyNull(rs.getString("source")),
-            emptyNull(rs.getString("server")),
-            rs.getDouble("q1"),
-            rs.getDouble("md"),
-            rs.getDouble("q3"),
+            getStringFromResultSet(rs, "source"),
+            getStringFromResultSet(rs, "server"),
+            getDoubleFromResultSet(rs, "q1"),
+            getDoubleFromResultSet(rs, "md"),
+            getDoubleFromResultSet(rs, "q3")));
+      }
+    }
+    return statistics;
+  }
+
+  static List<String> queryOnionperfFailures(Connection connection)
+      throws SQLException {
+    logger.info("Querying timeout and failure statistics from database.");
+    List<String> statistics = new ArrayList<>();
+    statistics.add("date,source,server,timeouts,failures,requests");
+    Statement st = connection.createStatement();
+    String queryString = "SELECT date, source, server, timeouts, failures, "
+        + "requests FROM onionperf_failures";
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
+    try (ResultSet rs = st.executeQuery(queryString)) {
+      while (rs.next()) {
+        statistics.add(String.format("%s,%s,%s,%d,%d,%d",
+            dateFormat.format(rs.getDate("date")),
+            getStringFromResultSet(rs, "source"),
+            getStringFromResultSet(rs, "server"),
             rs.getInt("timeouts"),
             rs.getInt("failures"),
             rs.getInt("requests")));
@@ -263,20 +306,18 @@ public class Main {
 
   static List<String> queryBuildTimes(Connection connection)
       throws SQLException {
-    log.info("Querying buildtime statistics from database.");
+    logger.info("Querying buildtime statistics from database.");
     List<String> statistics = new ArrayList<>();
     statistics.add("date,source,position,q1,md,q3");
     Statement st = connection.createStatement();
     String queryString = "SELECT date, source, position, q1, md, q3 "
         + "FROM buildtimes_stats";
-    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
-    dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
-    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
     try (ResultSet rs = st.executeQuery(queryString)) {
       while (rs.next()) {
         statistics.add(String.format("%s,%s,%d,%d,%d,%d",
-            dateFormat.format(rs.getDate("date", calendar)),
-            emptyNull(rs.getString("source")),
+            dateFormat.format(rs.getDate("date")),
+            getStringFromResultSet(rs, "source"),
             rs.getInt("position"),
             rs.getInt("q1"),
             rs.getInt("md"),
@@ -288,44 +329,84 @@ public class Main {
 
   static List<String> queryLatencies(Connection connection)
       throws SQLException {
-    log.info("Querying latency statistics from database.");
+    logger.info("Querying latency statistics from database.");
     List<String> statistics = new ArrayList<>();
-    statistics.add("date,source,server,q1,md,q3");
+    statistics.add("date,source,server,low,q1,md,q3,high");
     Statement st = connection.createStatement();
-    String queryString = "SELECT date, source, server, q1, md, q3 "
+    String queryString = "SELECT date, source, server, low, q1, md, q3, high "
         + "FROM latencies_stats";
-    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
-    dateFormat.setTimeZone(TimeZone.getTimeZone("UTC"));
-    Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
     try (ResultSet rs = st.executeQuery(queryString)) {
       while (rs.next()) {
-        statistics.add(String.format("%s,%s,%s,%d,%d,%d",
-            dateFormat.format(rs.getDate("date", calendar)),
-            emptyNull(rs.getString("source")),
+        statistics.add(String.format("%s,%s,%s,%d,%d,%d,%d,%d",
+            dateFormat.format(rs.getDate("date")),
+            getStringFromResultSet(rs, "source"),
             rs.getString("server"),
+            rs.getInt("low"),
             rs.getInt("q1"),
             rs.getInt("md"),
-            rs.getInt("q3")));
+            rs.getInt("q3"),
+            rs.getInt("high")));
       }
     }
     return statistics;
   }
 
-  private static String emptyNull(String text) {
-    return null == text ? "" : text;
+  static List<String> queryThroughput(Connection connection)
+      throws SQLException {
+    logger.info("Querying throughput statistics from database.");
+    List<String> statistics = new ArrayList<>();
+    statistics.add("date,source,server,low,q1,md,q3,high");
+    Statement st = connection.createStatement();
+    String queryString = "SELECT date, source, server, low, q1, md, q3, high "
+        + "FROM throughput_stats";
+    DateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
+    try (ResultSet rs = st.executeQuery(queryString)) {
+      while (rs.next()) {
+        statistics.add(String.format("%s,%s,%s,%d,%d,%d,%d,%d",
+            dateFormat.format(rs.getDate("date")),
+            getStringFromResultSet(rs, "source"),
+            rs.getString("server"),
+            rs.getInt("low"),
+            rs.getInt("q1"),
+            rs.getInt("md"),
+            rs.getInt("q3"),
+            rs.getInt("high")));
+      }
+    }
+    return statistics;
+  }
+
+  /** Retrieves the {@code String} value of the designated column in the
+   * current row of the given {@code ResultSet} object, or returns the
+   * empty string if the retrieved value was {@code NULL}. */
+  private static String getStringFromResultSet(ResultSet rs, String columnLabel)
+      throws SQLException {
+    String result = rs.getString(columnLabel);
+    return null == result ? "" : result;
+  }
+
+  /** Retrieves the {@code double} value of the designated column in the
+   * current row of the given {@code ResultSet} object as a
+   * {@code Double} object, or {@code null} if the retrieved value was
+   * {@code NULL}. */
+  private static Double getDoubleFromResultSet(ResultSet rs, String columnLabel)
+      throws SQLException {
+    double result = rs.getDouble(columnLabel);
+    return rs.wasNull() ? null : result;
   }
 
   static void writeStatistics(Path webstatsPath, List<String> statistics)
       throws IOException {
     webstatsPath.toFile().getParentFile().mkdirs();
-    log.info("Writing {} lines to {}.", statistics.size(),
+    logger.info("Writing {} lines to {}.", statistics.size(),
         webstatsPath.toFile().getAbsolutePath());
     Files.write(webstatsPath, statistics, StandardCharsets.UTF_8);
   }
 
   private static void disconnectFromDatabase(Connection connection)
       throws SQLException {
-    log.info("Disconnecting from database.");
+    logger.info("Disconnecting from database.");
     connection.close();
   }
 }
